@@ -63,6 +63,7 @@ class ItemSync::Private
     void createLocalItem( const Item &item );
     void checkDone();
     void slotLocalListDone( KJob* );
+    void slotLocalDeleteDone( KJob* );
     void slotLocalChangeDone( KJob* );
     void execute();
     void processItems();
@@ -109,6 +110,9 @@ class ItemSync::Private
 
 void ItemSync::Private::createLocalItem( const Item & item )
 {
+  // don't try to do anything in error state
+  if ( q->error() )
+    return;
   mPendingJobs++;
   ItemCreateJob *create = new ItemCreateJob( item, mSyncCollection, subjobParent() );
   q->connect( create, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalChangeDone( KJob* ) ) );
@@ -200,6 +204,10 @@ void ItemSync::doStart()
 
 bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
 {
+  // we are in error state, better not change anything at all anymore
+  if ( error() )
+    return false;
+
   /*
    * We know that this item has changed (as it is part of the
    * incremental changed list), so we just put it into the
@@ -210,21 +218,22 @@ bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
 
   // Check whether the flags differ
   if ( storedItem.flags() != newItem.flags() ) {
-    kDebug( 5250 ) << "Stored flags "  << storedItem.flags()
-                    << "new flags " << newItem.flags();
+    kDebug() << "Stored flags "  << storedItem.flags()
+             << "new flags " << newItem.flags();
     return true;
   }
 
   // Check whether the new item contains unknown parts
-  QSet<QByteArray> missingParts = storedItem.loadedPayloadParts();
-  missingParts.subtract( newItem.loadedPayloadParts() );
+  QSet<QByteArray> missingParts = newItem.loadedPayloadParts();
+  missingParts.subtract( storedItem.loadedPayloadParts() );
   if ( !missingParts.isEmpty() )
     return true;
 
   // ### FIXME SLOW!!!
   // If the available part identifiers don't differ, check
   // whether the content of the payload differs
-  if ( storedItem.payloadData() != newItem.payloadData() )
+  if ( newItem.hasPayload()
+    && storedItem.payloadData() != newItem.payloadData() )
     return true;
 
   // check if remote attributes have been changed
@@ -240,14 +249,15 @@ bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
 
 void ItemSync::Private::slotLocalListDone( KJob * job )
 {
-  if ( job->error() )
-    return;
-
-  const Item::List list = static_cast<ItemFetchJob*>( job )->items();
-  foreach ( const Item &item, list ) {
-    mLocalItemsById.insert( item.id(), item );
-    mLocalItemsByRemoteId.insert( item.remoteId(), item );
-    mUnprocessedLocalItems.insert( item );
+  if ( !job->error() ) {
+    const Item::List list = static_cast<ItemFetchJob*>( job )->items();
+    foreach ( const Item &item, list ) {
+      if ( item.remoteId().isEmpty() )
+        continue;
+      mLocalItemsById.insert( item.id(), item );
+      mLocalItemsByRemoteId.insert( item.remoteId(), item );
+      mUnprocessedLocalItems.insert( item );
+    }
   }
 
   mLocalListDone = true;
@@ -262,7 +272,8 @@ void ItemSync::Private::execute()
   if ( (mTransactionMode == Single && !mCurrentTransaction) || mTransactionMode == Chunkwise ) {
     ++mTransactionJobs;
     mCurrentTransaction = new TransactionSequence( q );
-    connect( mCurrentTransaction, SIGNAL(result(KJob*)), q, SLOT(slotTransactionResult(KJob*)) );
+    mCurrentTransaction->setAutomaticCommittingEnabled( false );
+    connect( mCurrentTransaction, SIGNAL( result( KJob* ) ), q, SLOT( slotTransactionResult( KJob* ) ) );
   }
 
   processItems();
@@ -299,7 +310,7 @@ void ItemSync::Private::processItems()
   foreach ( Item remoteItem, mRemoteItems ) { //krazy:exclude=foreach non-const is needed here
 #ifndef NDEBUG
     if ( remoteItem.remoteId().isEmpty() ) {
-      kWarning( 5250 ) << "Item " << remoteItem.id() << " does not have a remote identifier";
+      kWarning() << "Item " << remoteItem.id() << " does not have a remote identifier";
     }
 #endif
 
@@ -331,6 +342,11 @@ void ItemSync::Private::processItems()
 
 void ItemSync::Private::deleteItems( const Item::List &items )
 {
+  // if in error state, better not change anything anymore
+  if ( q->error() )
+    return;
+
+  Item::List itemsToDelete;
   foreach ( const Item &item, items ) {
     Item delItem( item );
     if ( !item.isValid() ) {
@@ -339,24 +355,47 @@ void ItemSync::Private::deleteItems( const Item::List &items )
 
     if ( !delItem.isValid() ) {
 #ifndef NDEBUG
-      kWarning( 5250 ) << "Delete item (remoteeId=" << delItem.remoteId()
-                       << "mimeType=" << delItem.mimeType()
-                       << ") does not have a valid UID and no item with that remote ID exists either";
+      kWarning() << "Delete item (remoteeId=" << delItem.remoteId()
+                 << "mimeType=" << delItem.mimeType()
+                 << ") does not have a valid UID and no item with that remote ID exists either";
 #endif
       continue;
     }
 
-    mPendingJobs++;
-    ItemDeleteJob *job = new ItemDeleteJob( delItem, subjobParent() );
-    q->connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalChangeDone( KJob* ) ) );
+    if ( delItem.remoteId().isEmpty() ) {
+      // don't attempt to remove items that never were written to the backend
+      continue;
+    }
+
+    itemsToDelete.append ( delItem );
   }
+
+  if ( !itemsToDelete.isEmpty() ) {
+    mPendingJobs++;
+    ItemDeleteJob *job = new ItemDeleteJob( itemsToDelete, subjobParent() );
+    q->connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalDeleteDone( KJob* ) ) );
+
+    // It can happen that the groupware servers report us deleted items
+    // twice, in this case this item delete job will fail on the second try.
+    // To avoid a rollback of the complete transaction we gracefully allow the job
+    // to fail :)
+    TransactionSequence *transaction = qobject_cast<TransactionSequence*>( subjobParent() );
+    if ( transaction )
+      transaction->setIgnoreJobFailure( job );
+  }
+}
+
+void ItemSync::Private::slotLocalDeleteDone( KJob* )
+{
+  mPendingJobs--;
+  mProgress++;
+
+  checkDone();
 }
 
 void ItemSync::Private::slotLocalChangeDone( KJob * job )
 {
-  if ( job->error() )
-    return;
-
+  Q_UNUSED( job );
   mPendingJobs--;
   mProgress++;
 
@@ -365,9 +404,6 @@ void ItemSync::Private::slotLocalChangeDone( KJob * job )
 
 void ItemSync::Private::slotTransactionResult( KJob *job )
 {
-  if ( job->error() )
-    return;
-
   --mTransactionJobs;
   if ( mCurrentTransaction == job )
     mCurrentTransaction = 0;
@@ -394,4 +430,29 @@ void ItemSync::deliveryDone()
   d->execute();
 }
 
+void ItemSync::slotResult(KJob* job)
+{
+  if ( job->error() ) {
+    // pretent there were no errors
+    Akonadi::Job::removeSubjob( job );
+    // propagate the first error we got but continue, we might still be fed with stuff from a resource
+    if ( !error() ) {
+      setError( job->error() );
+      setErrorText( job->errorText() );
+    }
+  } else {
+    Akonadi::Job::slotResult( job );
+  }
+}
+
+void ItemSync::rollback()
+{
+  setError( UserCanceled );
+  if ( d->mCurrentTransaction )
+    d->mCurrentTransaction->rollback();
+  d->mDeliveryDone = true; // user wont deliver more data
+  d->execute(); // end this in an ordered way, since we have an error set no real change will be done
+}
+
+
 #include "itemsync.moc"
