@@ -48,8 +48,8 @@ MonitorPrivate::MonitorPrivate(Monitor * parent) :
 
 void MonitorPrivate::init()
 {
-  QObject::connect( &collectionCache, SIGNAL(dataAvailable()), q_ptr, SLOT(dataAvailable()) );
-  QObject::connect( &itemCache, SIGNAL(dataAvailable()), q_ptr, SLOT(dataAvailable()) );
+  QObject::connect( &collectionCache, SIGNAL( dataAvailable() ), q_ptr, SLOT( dataAvailable() ) );
+  QObject::connect( &itemCache, SIGNAL( dataAvailable() ), q_ptr, SLOT( dataAvailable() ) );
 }
 
 bool MonitorPrivate::connectToNotificationManager()
@@ -66,8 +66,8 @@ bool MonitorPrivate::connectToNotificationManager()
   if ( !nm ) {
     kWarning() << "Unable to connect to notification manager";
   } else {
-    QObject::connect( nm, SIGNAL(notify(Akonadi::NotificationMessage::List)),
-             q_ptr, SLOT(slotNotify(Akonadi::NotificationMessage::List)) );
+    QObject::connect( nm, SIGNAL( notify( Akonadi::NotificationMessage::List ) ),
+                      q_ptr, SLOT( slotNotify( Akonadi::NotificationMessage::List ) ) );
     return true;
   }
   return false;
@@ -82,12 +82,13 @@ bool MonitorPrivate::isLazilyIgnored( const NotificationMessage & msg ) const
 {
   NotificationMessage::Operation op = msg.operation();
 
-  if ( ( msg.type() == NotificationMessage::Item ) && ( ( op == NotificationMessage::Add && q_ptr->receivers( SIGNAL(itemAdded(const Akonadi::Item &,const Akonadi::Collection &)) ) == 0 )
-    || ( op == NotificationMessage::Remove && q_ptr->receivers( SIGNAL(itemRemoved(const Akonadi::Item &)) ) == 0 )
-    || ( op == NotificationMessage::Modify && q_ptr->receivers( SIGNAL(itemChanged(const Akonadi::Item &,const QSet<QByteArray> &)) ) == 0 )
-    || ( op == NotificationMessage::Move && q_ptr->receivers( SIGNAL(itemMoved(const Akonadi::Item &, const Akonadi::Collection &, const Akonadi::Collection &)) ) == 0 )
-    || ( op == NotificationMessage::Link && q_ptr->receivers( SIGNAL(itemLinked(const Akonadi::Item &,const Akonadi::Collection &)) ) == 0 )
-    || ( op == NotificationMessage::Unlink && q_ptr->receivers( SIGNAL(itemUnlinked(const Akonadi::Item &,const Akonadi::Collection &)) ) == 0 ) ) )
+  if ( !fetchCollectionStatistics &&
+       ( msg.type() == NotificationMessage::Item ) && ( ( op == NotificationMessage::Add && q_ptr->receivers( SIGNAL( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ) ) == 0 )
+    || ( op == NotificationMessage::Remove && q_ptr->receivers( SIGNAL( itemRemoved( const Akonadi::Item& ) ) ) == 0 )
+    || ( op == NotificationMessage::Modify && q_ptr->receivers( SIGNAL( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ) ) == 0 )
+    || ( op == NotificationMessage::Move && q_ptr->receivers( SIGNAL( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ) ) == 0 )
+    || ( op == NotificationMessage::Link && q_ptr->receivers( SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ) ) == 0 )
+    || ( op == NotificationMessage::Unlink && q_ptr->receivers( SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ) ) == 0 ) ) )
   {
     return true;
   }
@@ -138,11 +139,13 @@ bool MonitorPrivate::acceptNotification( const NotificationMessage & msg )
     case NotificationMessage::Item:
       return isItemMonitored( msg.uid(), msg.parentCollection(), msg.parentDestCollection(), msg.mimeType(), msg.resource() )
           || isCollectionMonitored( msg.parentCollection(), msg.resource() )
-          || isCollectionMonitored( msg.parentDestCollection(), msg.resource() );
+          || isCollectionMonitored( msg.parentDestCollection(), msg.resource() )
+          || isMoveDestinationResourceMonitored( msg );
     case NotificationMessage::Collection:
       return isCollectionMonitored( msg.uid(), msg.resource() )
           || isCollectionMonitored( msg.parentCollection(), msg.resource() )
-          || isCollectionMonitored( msg.parentDestCollection(), msg.resource() );
+          || isCollectionMonitored( msg.parentDestCollection(), msg.resource() )
+          || isMoveDestinationResourceMonitored( msg );
   }
   Q_ASSERT( false );
   return false;
@@ -159,6 +162,21 @@ void MonitorPrivate::dispatchNotifications()
   }
 }
 
+void MonitorPrivate::cleanOldNotifications()
+{
+  foreach ( const NotificationMessage &msg, pipeline ) {
+    if ( !acceptNotification( msg ) ) {
+      pipeline.removeOne( msg );
+    }
+  }
+
+  foreach ( const NotificationMessage &msg, pendingNotifications ) {
+    if ( !acceptNotification( msg ) ) {
+      pendingNotifications.removeOne( msg );
+    }
+  }
+}
+
 bool MonitorPrivate::ensureDataAvailable( const NotificationMessage &msg )
 {
   bool allCached = true;
@@ -202,8 +220,10 @@ void MonitorPrivate::dataAvailable()
   while ( !pipeline.isEmpty() ) {
     const NotificationMessage msg = pipeline.head();
     if ( ensureDataAvailable( msg ) ) {
-      emitNotification( msg );
+      // dequeue should be before emit, otherwise stuff might happen (like dataAvailable
+      // being called again) and we end up dequeuing an empty pipeline
       pipeline.dequeue();
+      emitNotification( msg );
     } else {
       break;
     }
@@ -215,6 +235,8 @@ void MonitorPrivate::updatePendingStatistics( const NotificationMessage &msg )
 {
   if ( msg.type() == NotificationMessage::Item ) {
     notifyCollectionStatisticsWatchers( msg.parentCollection(), msg.resource() );
+    // FIXME use the proper resource of the target collection, for cross resource moves
+    notifyCollectionStatisticsWatchers( msg.parentDestCollection(), msg.resource() );
   } else if ( msg.type() == NotificationMessage::Collection && msg.operation() == NotificationMessage::Remove ) {
     // no need for statistics updates anymore
     recentlyChangedCollections.remove( msg.uid() );
@@ -223,9 +245,9 @@ void MonitorPrivate::updatePendingStatistics( const NotificationMessage &msg )
 
 void MonitorPrivate::slotSessionDestroyed( QObject * object )
 {
-  Session* session = qobject_cast<Session*>( object );
-  if ( session )
-    sessions.removeAll( session->sessionId() );
+  Session* objectSession = qobject_cast<Session*>( object );
+  if ( objectSession )
+    sessions.removeAll( objectSession->sessionId() );
 }
 
 void MonitorPrivate::slotStatisticsChangedFinished( KJob* job )
@@ -234,6 +256,7 @@ void MonitorPrivate::slotStatisticsChangedFinished( KJob* job )
     kWarning() << "Error on fetching collection statistics: " << job->errorText();
   } else {
     CollectionStatisticsJob *statisticsJob = static_cast<CollectionStatisticsJob*>( job );
+    Q_ASSERT( statisticsJob->collection().isValid() );
     emit q_ptr->collectionStatisticsChanged( statisticsJob->collection().id(),
                                              statisticsJob->statistics() );
   }
@@ -241,7 +264,8 @@ void MonitorPrivate::slotStatisticsChangedFinished( KJob* job )
 
 void MonitorPrivate::slotFlushRecentlyChangedCollections()
 {
-  foreach( Collection::Id collection, recentlyChangedCollections ) {
+  foreach ( Collection::Id collection, recentlyChangedCollections ) {
+    Q_ASSERT( collection >= 0 );
     if ( fetchCollectionStatistics ) {
       fetchStatistics( collection );
     } else {
@@ -284,8 +308,8 @@ void MonitorPrivate::slotNotify( const NotificationMessage::List &msgs )
 {
   foreach ( const NotificationMessage &msg, msgs ) {
     invalidateCaches( msg );
+    updatePendingStatistics( msg );
     if ( acceptNotification( msg ) ) {
-      updatePendingStatistics( msg );
       appendAndCompress( msg );
     }
   }
@@ -305,13 +329,19 @@ void MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const
   }
   if ( !colDest.isValid() ) {
     colDest = Collection( msg.parentDestCollection() );
-    // FIXME setResource here required ?
+    // HACK: destination resource is delivered in the parts field...
+    if ( !msg.itemParts().isEmpty() )
+      colDest.setResource( QString::fromLatin1( *(msg.itemParts().begin()) ) );
   }
   Item it = item;
   if ( !it.isValid() || msg.operation() == NotificationMessage::Remove ) {
     it = Item( msg.uid() );
     it.setRemoteId( msg.remoteId() );
     it.setMimeType( msg.mimeType() );
+  } else if ( it.remoteId().isEmpty() && !msg.remoteId().isEmpty() ) {
+    // recover RID, in case of inter-resource moves the source RID is only in the
+    // notification but not in the item loaded from Akonadi
+    it.setRemoteId( msg.remoteId() );
   }
   if ( !it.parentCollection().isValid() ) {
     if ( msg.operation() == NotificationMessage::Move )
@@ -319,6 +349,16 @@ void MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const
     else
       it.setParentCollection( col );
   }
+
+  // HACK: We have the remoteRevision stored in the itemParts set
+  //       for delete operations to avoid protocol breakage
+  if ( msg.operation() == NotificationMessage::Remove ) {
+    if ( !msg.itemParts().isEmpty() ) { //TODO: investigate why it could be empty
+      const QString remoteRevision = QString::fromUtf8( msg.itemParts().toList().first() );
+      it.setRemoteRevision( remoteRevision );
+    }
+  }
+
   switch ( msg.operation() ) {
     case NotificationMessage::Add:
       emit q_ptr->itemAdded( it, col );
@@ -369,6 +409,15 @@ void MonitorPrivate::emitCollectionNotification( const NotificationMessage &msg,
       collection.setParentCollection( parent );
   }
 
+  // HACK: We have the remoteRevision stored in the itemParts set
+  //       for delete operations to avoid protocol breakage
+  if ( msg.operation() == NotificationMessage::Remove ) {
+    if ( !msg.itemParts().isEmpty() ) { //TODO: investigate why it could be empty
+      const QString remoteRevision = QString::fromUtf8( msg.itemParts().toList().first() );
+      collection.setRemoteRevision( remoteRevision );
+    }
+  }
+
   switch ( msg.operation() ) {
     case NotificationMessage::Add:
       emit q_ptr->collectionAdded( collection, parent );
