@@ -30,7 +30,7 @@
 #include <akonadi/item.h>
 
 class KJob;
-class ResourceAdaptor;
+class Akonadi__ResourceAdaptor;
 
 namespace Akonadi {
 
@@ -101,6 +101,9 @@ class ResourceBasePrivate;
  * - itemAdded()
  * - itemChanged()
  * - itemRemoved()
+ * Note that these three functions don't get the full payload of the items by default,
+ * you need to change the item fetch scope of the change recorder to fetch the full
+ * payload. This can be expensive with big payloads, though.<br>
  * Once you have handled changes in these methods call changeCommitted().
  * These methods are called whenever a local item related to this resource is
  * added, modified or deleted. They are only called if the resource is online, otherwise
@@ -179,13 +182,13 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
       Observer *observer = dynamic_cast<Observer*>( r );
       if ( observer != 0 )
         r->registerObserver( observer );
+
       return init( r );
     }
 
     /**
      * This method is used to set the name of the resource.
      */
-    //FIXME_API: make sure location is renamed to this by resourcebase
     void setName( const QString &name );
 
     /**
@@ -201,6 +204,11 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void nameChanged( const QString &name );
 
+    /**
+     * Emitted when a full synchronization has been completed.
+     */
+    void synchronized();
+
   protected Q_SLOTS:
     /**
      * Retrieve the collection tree from the remote server and supply it via
@@ -216,19 +224,19 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      * If incremental retrieval is not possible, provide the full listing by calling
      * itemsRetrieved( const Item::List& ).
      * In any case, ensure that all items have a correctly set remote identifier
-     * to allow synchronizing with already locally existing items.
-     * In case you don't want to use the built-in item syncing code, store the retrived
+     * to allow synchronizing with items already existing locally.
+     * In case you don't want to use the built-in item syncing code, store the retrieved
      * items manually and call itemsRetrieved() once you are done.
-     * @param collection The collection to sync.
-     * @see itemsRetrieved( const Item::List &), itemsRetrievedIncremental(), itemsRetrieved(), currentCollection()
+     * @param collection The collection whose items to retrieve.
+     * @see itemsRetrieved( const Item::List& ), itemsRetrievedIncremental(), itemsRetrieved(), currentCollection()
      */
     virtual void retrieveItems( const Akonadi::Collection &collection ) = 0;
 
     /**
      * Retrieve a single item from the backend. The item to retrieve is provided as @p item.
      * Add the requested payload parts and call itemRetrieved() when done.
-     * @param item The empty item which payload should be retrieved. Use this object when delivering
-     * the result instead of creating a new item to ensure conflict detection to work.
+     * @param item The empty item whose payload should be retrieved. Use this object when delivering
+     * the result instead of creating a new item to ensure conflict detection will work.
      * @param parts The item parts that should be retrieved.
      * @return false if there is an immediate error when retrieving the item.
      * @see itemRetrieved()
@@ -293,6 +301,25 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
     void collectionsRetrievedIncremental( const Collection::List &changedCollections,
                                           const Collection::List &removedCollections );
 
+    /**
+     * Enable collection streaming, that is collections don't have to be delivered at once
+     * as result of a retrieveCollections() call but can be delivered by multiple calls
+     * to collectionsRetrieved() or collectionsRetrievedIncremental(). When all collections
+     * have been retrieved, call collectionsRetrievalDone().
+     * @param enable @c true if collection streaming should be enabled, @c false by default
+     */
+    void setCollectionStreamingEnabled( bool enable );
+
+    /**
+     * Call this method to indicate you finished synchronizing the collection tree.
+     *
+     * This is not needed if you use the built in syncing without collection streaming
+     * and call collectionsRetrieved() or collectionRetrievedIncremental() instead.
+     * If collection streaming is enabled, call this method once all collections have been delivered
+     * using collectionsRetrieved() or collectionsRetrievedIncremental().
+     */
+    void collectionsRetrievalDone();
+
     /**
      * Call this method to supply the full collection listing from the remote server.
      *
@@ -339,6 +366,17 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void itemsRetrievalDone();
 
+    /**
+     * Call this method to remove all items and collections of the resource from the
+     * server cache.
+     *
+     * The method should be used whenever the configuration of the resource has changed
+     * and therefor the cached items might not be valid any longer.
+     *
+     * @since 4.3
+     */
+    void clearCache();
+
     /**
      * Returns the collection that is currently synchronized.
      */
@@ -376,17 +414,77 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void cancelTask( const QString &error );
 
+    /**
+     * Stops the execution of the current task and continues with the next one.
+     * The current task will be tried again later.
+     *
+     * @since 4.3
+     */
+    void deferTask();
+
     /**
      * Inherited from AgentBase.
      */
     void doSetOnline( bool online );
 
+    /**
+     * Indicate the use of hierarchical remote identifiers.
+     *
+     * This means that it is possible to have two different items with the same
+     * remoteId in different Collections.
+     *
+     * This should be called in the resource constructor as needed.
+     *
+     * @since 4.4
+     */
+    void setHierarchicalRemoteIdentifiersEnabled( bool enable );
+
+    friend class ResourceScheduler;
+
+    /**
+     * Describes the scheduling priority of a task that has been queued
+     * for execution.
+     *
+     * @see scheduleCustomTask
+     * @since 4.4
+     */
+    enum SchedulePriority {
+      Prepend,            ///> The task will be executed as soon as the current task has finished.
+      AfterChangeReplay,  ///> The task is scheduled after the last ChangeReplay task in the queue
+      Append              ///> The task will be executed after all tasks currently in the queue are finished
+    };
+
+    /**
+     * Schedules a custom task in the internal scheduler. It will be queued with
+     * all other tasks such as change replays and retrieval requests and eventually
+     * executed by calling the specified method. With the priority parameter the
+     * time of execution of the Task can be influenced. @see SchedulePriority
+     * @param receiver The object the slot should be called on.
+     * @param method The name of the method (and only the name, not signature, not SLOT(...) macro),
+     * that should be called to execute this task. The method has to be a slot and take a QVariant as
+     * argument.
+     * @param argument A QVariant argument passed to the method specified above. Use this to pass task
+     * parameters.
+     * @param priority Priority of the task. Use this to influence the position in
+     * the execution queue.
+     * @since 4.4
+     */
+    void scheduleCustomTask( QObject* receiver, const char* method, const QVariant &argument, SchedulePriority priority = Append );
+
+    /**
+     * Indicate that the current task is finished. Use this method from the slot called via scheduleCustomTaks().
+     * As with all the other callbacks, make sure to either call taskDone() or cancelTask()/deferTask() on all
+     * exit paths, otherwise the resource will hang.
+     * @since 4.4
+     */
+    void taskDone();
+
   private:
     static QString parseArguments( int, char** );
     static int init( ResourceBase *r );
 
     // dbus resource interface
-    friend class ::ResourceAdaptor;
+    friend class ::Akonadi__ResourceAdaptor;
 
     bool requestItemDelivery( qint64 uid, const QString &remoteId, const QString &mimeType, const QStringList &parts );
 
@@ -395,11 +493,17 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
 
     Q_PRIVATE_SLOT( d_func(), void slotDeliveryDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotCollectionSyncDone( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void slotDeleteResourceCollection() )
+    Q_PRIVATE_SLOT( d_func(), void slotDeleteResourceCollectionDone( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void slotCollectionDeletionDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotLocalListDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotSynchronizeCollection( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void slotCollectionListDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotItemSyncDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotPercent( KJob*, unsigned long ) )
+    Q_PRIVATE_SLOT( d_func(), void slotPrepareItemRetrieval( const Akonadi::Item& item ) )
+    Q_PRIVATE_SLOT( d_func(), void slotPrepareItemRetrievalResult( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void changeCommittedResult( KJob* ) )
 };
 
 }
