@@ -41,9 +41,9 @@
 #include "journal.h"
 
 extern "C" {
-  #include <ical.h>
-  #include <icalparser.h>
-  #include <icalrestriction.h>
+  #include <libical/ical.h>
+  #include <libical/icalparser.h>
+  #include <libical/icalrestriction.h>
 }
 
 #include <QtCore/QString>
@@ -881,8 +881,8 @@ icalrecurrencetype ICalFormatImpl::writeRecurrenceRule( RecurrenceRule *recur )
   bys = recur->byMonthDays();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    short d = static_cast<short>( ( *it ) * 8 );
-    r.by_month_day[index++] = static_cast<short>( icalrecurrencetype_day_position( d ) );
+    short dShort = static_cast<short>( ( *it ) * 8 );
+    r.by_month_day[index++] = static_cast<short>( icalrecurrencetype_day_position( dShort ) );
   }
 
   bys = recur->byYearDays();
@@ -1204,6 +1204,7 @@ FreeBusy *ICalFormatImpl::readFreeBusy( icalcomponent *vfreebusy )
   while ( p ) {
     icalproperty_kind kind = icalproperty_isa( p );
     switch ( kind ) {
+
     case ICAL_DTSTART_PROPERTY:  // start date and time (UTC)
       freebusy->setDtStart( readICalUtcDateTimeProperty( p ) );
       break;
@@ -1224,10 +1225,19 @@ FreeBusy *ICalFormatImpl::readFreeBusy( icalcomponent *vfreebusy )
         Duration duration ( readICalDuration( icalperiod.duration ) );
         period = FreeBusyPeriod( period_start, duration );
       }
-      QByteArray param = icalproperty_get_parameter_as_string( p, "X-SUMMARY" );
-      period.setSummary( QString::fromUtf8( KCodecs::base64Decode( param ) ) );
-      param = icalproperty_get_parameter_as_string( p, "X-LOCATION" );
-      period.setLocation( QString::fromUtf8( KCodecs::base64Decode( param ) ) );
+
+      icalparameter *param = icalproperty_get_first_parameter( p, ICAL_X_PARAMETER );
+      while ( param ) {
+        if ( strncmp( icalparameter_get_xname( param ), "X-SUMMARY", 9 ) == 0 ) {
+          period.setSummary( QString::fromUtf8(
+                               KCodecs::base64Decode( icalparameter_get_xvalue( param ) ) ) );
+        }
+        if ( strncmp( icalparameter_get_xname( param ), "X-LOCATION", 10 ) == 0 ) {
+          period.setLocation( QString::fromUtf8(
+                                KCodecs::base64Decode( icalparameter_get_xvalue( param ) ) ) );
+        }
+        param = icalproperty_get_next_parameter( p, ICAL_X_PARAMETER );
+      }
       periods.append( period );
       break;
     }
