@@ -42,6 +42,9 @@
 #include <QtGui/QKeyEvent>
 #include <QtGui/QTextLayout>
 
+#include "textutils.h"
+#include <QPlainTextEdit>
+
 namespace KPIMTextEdit {
 
 class TextEditPrivate
@@ -49,7 +52,9 @@ class TextEditPrivate
   public:
 
     TextEditPrivate( TextEdit *parent )
-      : q( parent ),
+      : actionAddImage( 0 ),
+        actionDeleteLine( 0 ),
+        q( parent ),
         imageSupportEnabled( false )
     {
     }
@@ -86,9 +91,14 @@ class TextEditPrivate
      */
     void _k_slotAddImage();
 
+    void _k_slotDeleteLine();
+
     /// The action that triggers _k_slotAddImage()
     KAction *actionAddImage;
 
+    /// The action that triggers _k_slotDeleteLine()
+    KAction *actionDeleteLine;
+
     /// The parent class
     TextEdit *q;
 
@@ -114,6 +124,8 @@ class TextEditPrivate
      * compatible way.
      */
     bool spellCheckingEnabled;
+
+    QString configFile;
 };
 
 } // namespace
@@ -147,6 +159,14 @@ TextEdit::TextEdit( QWidget *parent )
   d->init();
 }
 
+TextEdit::TextEdit( QWidget *parent, const QString& configFile )
+  : KRichTextWidget( parent ),
+    d( new TextEditPrivate( this ) )
+{
+  d->init();
+  d->configFile = configFile;
+}
+
 TextEdit::~TextEdit()
 {
 }
@@ -175,6 +195,12 @@ void TextEditPrivate::init()
   q->installEventFilter( q );
 }
 
+QString TextEdit::configFile() const
+{
+   return d->configFile;
+}
+
+
 void TextEdit::keyPressEvent ( QKeyEvent * e )
 {
   if ( e->key() ==  Qt::Key_Return ) {
@@ -326,11 +352,11 @@ QString TextEdit::toWrappedPlainText() const
     }
     block = block.next();
   }
-
+  
   // Remove the last superfluous newline added above
   if ( temp.endsWith( QLatin1Char( '\n' ) ) )
     temp.chop( 1 );
-
+  
   d->fixupTextEditString( temp );
   return temp;
 }
@@ -352,6 +378,11 @@ void TextEdit::createActions( KActionCollection *actionCollection )
     actionCollection->addAction( QLatin1String( "add_image" ), d->actionAddImage );
     connect( d->actionAddImage, SIGNAL(triggered(bool) ), SLOT( _k_slotAddImage() ) );
   }
+
+  d->actionDeleteLine = new KAction( i18n( "Delete Line" ), this );
+  d->actionDeleteLine->setShortcut( QKeySequence( Qt::CTRL + Qt::Key_K ) );
+  actionCollection->addAction( QLatin1String( "delete_line" ), d->actionDeleteLine );
+  connect( d->actionDeleteLine, SIGNAL(triggered(bool)), SLOT(_k_slotDeleteLine()) );
 }
 
 void TextEdit::addImage( const KUrl &url )
@@ -368,6 +399,38 @@ void TextEdit::addImage( const KUrl &url )
   d->addImageHelper( imageName, image );
 }
 
+void TextEdit::loadImage ( const QImage& image, const QString& matchName, const QString& resourceName )
+{
+  QSet<int> cursorPositionsToSkip;
+  QTextBlock currentBlock = document()->begin();
+  QTextBlock::iterator it;
+  while ( currentBlock.isValid() ) {
+    for (it = currentBlock.begin(); !(it.atEnd()); ++it) {
+      QTextFragment fragment = it.fragment();
+      if ( fragment.isValid() ) {
+        QTextImageFormat imageFormat = fragment.charFormat().toImageFormat();
+        if ( imageFormat.isValid() && imageFormat.name() == matchName ) {
+          int pos = fragment.position();
+          if ( !cursorPositionsToSkip.contains( pos ) ) {
+            QTextCursor cursor( document() );
+            cursor.setPosition( pos );
+            cursor.setPosition( pos + 1, QTextCursor::KeepAnchor );
+            cursor.removeSelectedText();
+            document()->addResource( QTextDocument::ImageResource, QUrl( resourceName ), QVariant( image ) );
+            cursor.insertImage( resourceName );
+
+            // The textfragment iterator is now invalid, restart from the beginning
+            // Take care not to replace the same fragment again, or we would be in an infinite loop.
+            cursorPositionsToSkip.insert( pos );
+            it = currentBlock.begin();
+          }
+        }
+      }
+    }
+    currentBlock = currentBlock.next();
+  }
+}
+
 void TextEditPrivate::addImageHelper( const QString &imageName, const QImage &image )
 {
   QString imageNameToAdd = imageName;
@@ -397,31 +460,45 @@ void TextEditPrivate::addImageHelper( const QString &imageName, const QImage &im
   q->enableRichTextMode();
 }
 
-QList< QSharedPointer<EmbeddedImage> > TextEdit::embeddedImages() const
+ImageWithNameList TextEdit::imagesWithName() const
 {
-  QList< QSharedPointer<EmbeddedImage> > retImages;
+  ImageWithNameList retImages;
   QStringList seenImageNames;
   QList<QTextImageFormat> imageFormats = d->embeddedImageFormats();
   foreach( const QTextImageFormat &imageFormat, imageFormats ) {
     if ( !seenImageNames.contains( imageFormat.name() ) ) {
-      QVariant data = document()->resource( QTextDocument::ImageResource, QUrl( imageFormat.name() ) );
-      QImage image = qvariant_cast<QImage>( data );
-      QBuffer buffer;
-      buffer.open( QIODevice::WriteOnly );
-      image.save( &buffer, "PNG" );
-
-      qsrand( QDateTime::currentDateTime().toTime_t() + qHash( imageFormat.name() ) );
-      QSharedPointer<EmbeddedImage> embeddedImage( new EmbeddedImage() );
-      retImages.append( embeddedImage );
-      embeddedImage->image = KMime::Codec::codecForName( "base64" )->encode( buffer.buffer() );
-      embeddedImage->imageName = imageFormat.name();
-      embeddedImage->contentID = QString( QLatin1String( "%1" ) ).arg( qrand() );
+      QVariant resourceData = document()->resource( QTextDocument::ImageResource, QUrl( imageFormat.name() ) );
+      QImage image = qvariant_cast<QImage>( resourceData );
+      QString name = imageFormat.name();
+      ImageWithNamePtr newImage( new ImageWithName );
+      newImage->image = image;
+      newImage->name = name;
+      retImages.append( newImage );
       seenImageNames.append( imageFormat.name() );
     }
   }
   return retImages;
 }
 
+QList< QSharedPointer<EmbeddedImage> > TextEdit::embeddedImages() const
+{
+  ImageWithNameList normalImages = imagesWithName();
+  QList< QSharedPointer<EmbeddedImage> > retImages;
+  foreach( const ImageWithNamePtr &normalImage, normalImages ) {
+    QBuffer buffer;
+    buffer.open( QIODevice::WriteOnly );
+    normalImage->image.save( &buffer, "PNG" );
+
+    qsrand( QDateTime::currentDateTime().toTime_t() + qHash( normalImage->name ) );
+    QSharedPointer<EmbeddedImage> embeddedImage( new EmbeddedImage() );
+    retImages.append( embeddedImage );
+    embeddedImage->image = KMime::Codec::codecForName( "base64" )->encode( buffer.buffer() );
+    embeddedImage->imageName = normalImage->name;
+    embeddedImage->contentID = QString( QLatin1String( "%1@KDE" ) ).arg( qrand() );
+  }
+  return retImages;
+}
+
 QList<QTextImageFormat> TextEditPrivate::embeddedImageFormats() const
 {
   QTextDocument *doc = q->document();
@@ -517,91 +594,59 @@ bool KPIMTextEdit::TextEdit::canInsertFromMimeData( const QMimeData *source ) co
   return KRichTextWidget::canInsertFromMimeData( source );
 }
 
-static bool isCharFormatFormatted( const QTextCharFormat &format, const QFont &defaultFont,
-                                   const QTextCharFormat &defaultBlockFormat )
-{
-  if ( !format.anchorHref().isEmpty() ||
-       format.font() != defaultFont ||
-       format.isAnchor() ||
-       format.verticalAlignment() != defaultBlockFormat.verticalAlignment() ||
-       format.underlineStyle() != defaultBlockFormat.underlineStyle() ||
-       format.foreground().color() != defaultBlockFormat.foreground().color() ||
-       format.background().color() != defaultBlockFormat.background().color() )
-    return true;
-
-  return false;
-}
-
-static bool isBlockFormatFormatted( const QTextBlockFormat &format,
-                                    const QTextBlockFormat &defaultFormat )
-{
-  if ( format.alignment() != defaultFormat.alignment() ||
-       format.indent() != defaultFormat.indent() ||
-       format.textIndent() != defaultFormat.textIndent() )
-    return true;
-
-  return false;
-}
-
-/// @return true if the format represents a list, table, image or something like that.
-static bool isSpecial( const QTextFormat &charFormat )
-{
-  return charFormat.isFrameFormat() || charFormat.isImageFormat() ||
-         charFormat.isListFormat() || charFormat.isTableFormat();
-}
-
 bool TextEdit::isFormattingUsed() const
 {
   if ( textMode() == Plain )
     return false;
 
-  // Below, we walk through all text blocks and through all text fragments in them
-  // and check if any of those has any formatting.
-  // To check if they have formatting, we use the functions isBlockFormatFormatted() and
-  // isCharFormatFormatted(). Those do not check all the exising formatting possibilities on
-  // earth, but everything that KRichTextEdit supports at the moment.
-  //
-  // Also, we have to compare the formats against those of a default text edit. For example,
-  // we can't compare the foreground color against black, because the user might have another
-  // color scheme. Therefore we compare the foreground color against a default text edit.
-
-  QTextEdit defaultTextEdit;
-  QTextCharFormat defaultCharFormat = defaultTextEdit.document()->begin().charFormat();
-  QTextBlockFormat defaultBlockFormat = defaultTextEdit.document()->begin().blockFormat();
-  QFont defaultFont = document()->defaultFont();
-
-  QTextBlock block = document()->firstBlock();
-  while ( block.isValid() ) {
-
-    if ( isBlockFormatFormatted( block.blockFormat(), defaultBlockFormat ) ) {
-      return true;
-    }
-
-    if ( isSpecial( block.charFormat() ) || isSpecial( block.blockFormat() ) ||
-         block.textList() ) {
-      return true;
-    }
-
-    QTextBlock::iterator it = block.begin();
-    while ( !it.atEnd() ) {
-      QTextFragment fragment = it.fragment();
-      QTextCharFormat charFormat = fragment.charFormat();
-      if ( isSpecial( charFormat ) ) {
-        return true;
-      }
-      if ( isCharFormatFormatted( fragment.charFormat(), defaultFont, defaultCharFormat ) ) {
-        return true;
-      }
-
-      it++;
-    }
-    block = block.next();
-  }
-
-  if ( toHtml().contains( QLatin1String( "<hr />" ) ) )
-    return true;
-
-  return false;
+  return TextUtils::containsFormatting( document() );
 }
 
+void TextEditPrivate::_k_slotDeleteLine()
+{
+  q->deleteCurrentLine();
+}
+
+void TextEdit::deleteCurrentLine()
+{
+  QTextCursor cursor = textCursor();
+  QTextBlock block = cursor.block();
+  const QTextLayout* layout = block.layout();
+
+  // The current text block can have several lines due to word wrapping.
+  // Search the line the cursor is in, and then delete it.
+  for ( int lineNumber = 0; lineNumber < layout->lineCount(); lineNumber++ ) {
+    QTextLine line = layout->lineAt( lineNumber );
+    const bool lastLineInBlock = ( line.textStart() + line.textLength() == block.length() - 1 );
+    const bool oneLineBlock = ( layout->lineCount() == 1 );
+    const int startOfLine = block.position() + line.textStart();
+    int endOfLine = block.position() + line.textStart() + line.textLength();
+    if ( !lastLineInBlock )
+      endOfLine -= 1;
+
+    // Found the line where the cursor is in
+    if ( cursor.position() >= startOfLine && cursor.position() <= endOfLine ) {
+      int deleteStart = startOfLine;
+      int deleteLength = line.textLength();
+      if ( oneLineBlock )
+        deleteLength++; // The trailing newline
+
+      // When deleting the last line in the document,
+      // remove the newline of the line before the last line instead
+      if ( deleteStart + deleteLength >= document()->characterCount() &&
+           deleteStart > 0 )
+        deleteStart--;
+
+      cursor.beginEditBlock();
+      cursor.setPosition( deleteStart );
+      cursor.movePosition( QTextCursor::NextCharacter, QTextCursor::KeepAnchor, deleteLength );
+      cursor.removeSelectedText();
+      cursor.endEditBlock();
+      return;
+    }
+  }
+
+}
+
+
 #include "textedit.moc"
