@@ -18,17 +18,22 @@
 */
 
 #include "transport.h"
-#include "transportmanager.h"
-#include "mailtransport_defs.h"
 #include "legacydecrypt.h"
+#include "mailtransport_defs.h"
+#include "transportmanager.h"
+#include "transporttype_p.h"
 
 #include <QTimer>
-#include <kdebug.h>
-#include <klocale.h>
-#include <kmessagebox.h>
-#include <kstringhandler.h>
-#include <kwallet.h>
-#include <kconfiggroup.h>
+
+#include <KConfigGroup>
+#include <KDebug>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KStringHandler>
+#include <KWallet/Wallet>
+
+#include <akonadi/agentinstance.h>
+#include <akonadi/agentmanager.h>
 
 using namespace MailTransport;
 using namespace KWallet;
@@ -40,6 +45,7 @@ using namespace KWallet;
 class TransportPrivate
 {
   public:
+    TransportType transportType;
     QString password;
     bool passwordLoaded;
     bool passwordDirty;
@@ -89,6 +95,24 @@ void Transport::setPassword( const QString &passwd )
   d->password = passwd;
 }
 
+void Transport::forceUniqueName()
+{
+  QStringList existingNames;
+  foreach ( Transport *t, TransportManager::self()->transports() ) {
+    if ( t->id() != id() ) {
+      existingNames << t->name();
+    }
+  }
+  int suffix = 1;
+  QString origName = name();
+  while ( existingNames.contains( name() ) ) {
+    setName( i18nc( "%1: name; %2: number appended to it to make "
+                    "it unique among a list of names", "%1 #%2", origName, suffix ) );
+    ++suffix;
+  }
+
+}
+
 void Transport::updatePasswordState()
 {
   Transport *original = TransportManager::self()->transportById( id(), false );
@@ -112,19 +136,30 @@ bool Transport::isComplete() const
 
 QString Transport::authenticationTypeString() const
 {
-  switch ( authenticationType() ) {
-  case EnumAuthenticationType::LOGIN:
-    return QLatin1String( "LOGIN" );
-  case EnumAuthenticationType::PLAIN:
-    return QLatin1String( "PLAIN" );
-  case EnumAuthenticationType::CRAM_MD5:
-    return QLatin1String( "CRAM-MD5" );
-  case EnumAuthenticationType::DIGEST_MD5:
-    return QLatin1String( "DIGEST-MD5" );
-  case EnumAuthenticationType::NTLM:
-    return QLatin1String( "NTLM" );
-  case EnumAuthenticationType::GSSAPI:
-    return QLatin1String( "GSSAPI" );
+  return Transport::authenticationTypeString( authenticationType() );
+}
+
+QString Transport::authenticationTypeString( int type )
+{
+  switch ( type ) {
+    case EnumAuthenticationType::LOGIN:
+      return QLatin1String( "LOGIN" );
+    case EnumAuthenticationType::PLAIN:
+      return QLatin1String( "PLAIN" );
+    case EnumAuthenticationType::CRAM_MD5:
+      return QLatin1String( "CRAM-MD5" );
+    case EnumAuthenticationType::DIGEST_MD5:
+      return QLatin1String( "DIGEST-MD5" );
+    case EnumAuthenticationType::NTLM:
+      return QLatin1String( "NTLM" );
+    case EnumAuthenticationType::GSSAPI:
+      return QLatin1String( "GSSAPI" );
+    case EnumAuthenticationType::CLEAR:
+      return i18nc( "Authentication method", "Clear text" );
+    case EnumAuthenticationType::APOP:
+      return QLatin1String( "APOP" );
+    case EnumAuthenticationType::ANONYMOUS:
+      return i18nc( "Authentication method", "Anonymous" );
   }
   Q_ASSERT( false );
   return QString();
@@ -140,6 +175,32 @@ void Transport::usrReadConfig()
     d->oldName = name();
   }
 
+  // Set TransportType.
+  {
+    using namespace Akonadi;
+    d->transportType = TransportType();
+    d->transportType.d->mType = type();
+    kDebug() << "type" << type();
+    if ( type() == EnumType::Akonadi ) {
+      const AgentInstance instance = AgentManager::self()->instance( host() );
+      if ( !instance.isValid() ) {
+        kWarning() << "Akonadi transport with invalid resource instance.";
+      }
+      d->transportType.d->mAgentType = instance.type();
+      kDebug() << "agent type" << instance.type().name() << "id" << instance.type().identifier();
+    }
+    // Now we have the type and possibly agentType.  Get the name, description
+    // etc. from TransportManager.
+    const TransportType::List &types = TransportManager::self()->types();
+    int index = types.indexOf( d->transportType );
+    if ( index != -1 ) {
+      d->transportType = types[ index ];
+    } else {
+      kWarning() << "Type unknown to manager.";
+      d->transportType.d->mName = i18nc( "An unknown transport type", "Unknown" );
+    }
+  }
+
   // we have everything we need
   if ( !storePassword() || d->passwordLoaded ) {
     return;
@@ -272,4 +333,19 @@ Transport *Transport::clone() const
   return new Transport( id );
 }
 
+TransportType Transport::transportType() const
+{
+  if ( !d->transportType.isValid() ) {
+    kWarning() << "Invalid transport type.";
+  }
+  return d->transportType;
+}
+
+void Transport::setTransportType( const TransportType &type )
+{
+  Q_ASSERT( type.isValid() );
+  d->transportType = type;
+  setType( type.type() );
+}
+
 #include "transport.moc"
